# Python imports
import subprocess

# Lib imports
from gi.repository import GLib

# Application imports
# from libs import websockets
from libs.websocket_client import WebsocketClient
from libs.dto.lsp_messages import LEN_HEADER, TYPE_HEADER, get_message_str, get_message_obj
from libs.dto.lsp_message_structs import \
    LSPResponseTypes, ClientRequest, ClientNotification, LSPResponseRequest, LSPResponseNotification
from .lsp_controller_base import LSPControllerBase



class LSPControllerWebsocket(LSPControllerBase):
    def _send_message(self, data: ClientRequest or ClientNotification):
        if not data or not hasattr(self, "lsp_process"): return

        message_str  = get_message_str(data)
        message_size = len(message_str)
        message      = f"Content-Length: {message_size}\r\n\r\n{message_str}"

        self.log_list.add_log_entry("Client", data)
        self.ws_client.send(message_str)

    def start_lsp(self):
        if not self._start_command: return
        try:
            self.lsp_process = subprocess.Popen(
                ["pylsp", "--ws", "--port", "4114"],
                stdout = subprocess.PIPE,
                stdin  = subprocess.PIPE
            )
        except Exception as e:
            self.log_list.add_log_entry(
                "LSP Client Error",
                LSPResponseRequest(
                    "2.0",
                    None,
                    {
                        "error": repr(e)
                    }
                )
            )

            return

        self.ws_client = WebsocketClient()
        self.ws_client.set_callback(self._monitor_lsp_response)
        self.ws_client.start_client()
        return self.lsp_process.pid

    def stop_lsp(self):
        if self._lsp_pid == -1: return

        self._lsp_pid    = -1
        self._message_id = 0
        self.ws_client.close_client()
        self.lsp_process.terminate()

    def _monitor_lsp_response(self, data: None or {}):
        if not hasattr(self, "lsp_process"): return
        if not data: return

        message      = get_message_obj(data)
        keys         = message.keys()
        lsp_response = None

        if "result" in keys:
            lsp_response = LSPResponseRequest(**get_message_obj(data))

        if "method" in keys:
            lsp_response = LSPResponseNotification(**get_message_obj(data))

        response_id  = -1

        if not lsp_response: return
        self.handle_lsp_response(lsp_response)